print("Importing Camera Libraries...", end=" ")
from picamera.array import PiRGBArray
from picamera import PiCamera
import time
import cv2
import queue
import threading
from datetime import datetime

print("[DONE]")

resolution = (1024,768)


class Camera:
    
    def __init__(self):
        print("Initializing Camera...", end=" ")
        self.camera = PiCamera()
        self.camera.resolution = resolution
        self.camera.framerate = 32
        self.rawCapture = PiRGBArray(self.camera, size=resolution)
        self._terminate = False
        time.sleep(0.1)
        
        self.q = queue.Queue()
        t = threading.Thread(target=self._reader)
        t.daemon = True
        t.start()
        print("[DONE]")
        
        
    def _reader(self):
        for frame in self.camera.capture_continuous(self.rawCapture, format="bgr", use_video_port=True):
            image = frame.array
            
            if not self.q.empty():
                try:
                    self.q.get_nowait()  # discard previous frame
                except queue.Empty:
                    pass
                
            self.q.put(image)
            self.rawCapture.truncate(0)
            
            if self._terminate:
                break            
    
    def read(self):
        return self.q.get()
    
    def close(self):
        self._terminate = True
        time.sleep(0.1)
        self.camera.close()


if __name__ == "__main__":
    cam = Camera()
    while True:    
        frame = cam.read()
        cv2.imshow("frame", cv2.resize(frame, (640,480)))
        if chr(cv2.waitKey(1)&255) == 's':
            now = datetime.now()
            img_str = now.strftime("%Y%m%d_%H%M%S")+".jpg"
            cv2.imwrite(img_str, frame)
            print("Image saved")
        elif chr(cv2.waitKey(1)&255) == 'q':
            cv2.destroyAllWindows()
            cam.close()
            break